-- LuxuryResourceDisplay.lua
-- Author: magus424
-- DateCreated: 9/29/2010 12:12:12 AM
--------------------------------------------------------------

-- print("Initializing...");

function DebugObject(o)
	for key,value in pairs(o) do
		if (value == true) then
			print(string.format("o.%s: %s", key, "true"));
		elseif (value == false) then
			print(string.format("o.%s: %s", key, "false"));
		elseif (type(value) == "string" or type(value) == "integer") then
			print(string.format("o.%s: %s", key, value));
		else
			print(string.format("o.%s: %s", key, "unknown"));
		end
	end
end

function TradedResources(iResourceID)
	local iPlayerID = Game.GetActivePlayer();
	local count = 0;

	if(iPlayerID >= 0) then
		local itemType;
		local duration;
		local finalTurn;
		local data1;
		local data2;
		local fromPlayer;

		local m_Deal = UI.GetScratchDeal();
		local iNumCurrentDeals = UI:GetNumCurrentDeals( iPlayer );

		if( iNumCurrentDeals > 0 ) then
			for i = 0, iNumCurrentDeals - 1 do
				UI.LoadCurrentDeal( iPlayerID, i );
				
				m_Deal:ResetIterator();
				itemType, duration, finalTurn, data1, data2, fromPlayer = m_Deal:GetNextItem();
				repeat
					if (data1 == iResourceID) then
						if (fromPlayer == iPlayerID) then
							count = count - 1;
						else
							count = count + 1;
						end
					end
					itemType, duration, finalTurn, data1, data2, fromPlayer = m_Deal:GetNextItem();
				until (itemType == nil);
			end
		end
	end
	return count;
end

function UpdateLuxuryResourceDisplay()
	local iPlayerID = Game.GetActivePlayer();

	if(iPlayerID >= 0) then
		local pPlayer = Players[iPlayerID];

		if (pPlayer:GetNumCities() > 0) then
			-----------------------------
			-- Update Resources
			-----------------------------
			local pResource;
			local iNumAvailable;
			local iNumUsed;
			local iNumTotal;

			local iTotal = 0;

			for pResource in GameInfo.Resources() do
				local iResourceLoop = pResource.ID;
				local bLuxury = (Game.GetResourceUsageType(iResourceLoop) == ResourceUsageTypes.RESOURCEUSAGE_LUXURY);

				if (bLuxury) then
					iNumAvailable = pPlayer:GetNumResourceAvailable(iResourceLoop, true);

					if (iNumAvailable > 0) then
						iTotal = iTotal + 1;
					end
				end
			end

			Controls.LuxuryResourceString:SetHide(false);
			Controls.LuxuryResourceString:SetText(string.format("Luxuries (%i)", iTotal));
		else
			Controls.LuxuryResourceString:SetHide(true);
		end
	end
end


-------------------------------------------------
-- TOOLTIPS
-------------------------------------------------


-- Tooltip init
function DoInitTooltipsLR()
	Controls.LuxuryResourceString:SetToolTipCallback(LuxuryResourcesTipHandler);
end

local tipControlTable = {};
TTManager:GetTypeControlTable("TooltipTypeTopPanel", tipControlTable);

-- Resources Tooltip
function LuxuryResourcesTipHandler(control)
	local strText;
	local strExtraText;
	local iPlayerID = Game.GetActivePlayer();
	local pPlayer = Players[iPlayerID];

	local pResource;
	local iNumAvailable;
	local iNumUsed;
	local iNumTotal;

	local tKeys = {};
	local tResources = {};

	for pResource in GameInfo.Resources() do
		local iResourceLoop = pResource.ID;
		local bLuxury = (Game.GetResourceUsageType(iResourceLoop) == ResourceUsageTypes.RESOURCEUSAGE_LUXURY);

		if (bLuxury) then
			iNumAvailable = pPlayer:GetNumResourceAvailable(iResourceLoop, true);
			iNumUsed = pPlayer:GetNumResourceUsed(iResourceLoop);
			iNumTotal = pPlayer:GetNumResourceTotal(iResourceLoop, true);

			strText = "";
			strExtraText = "";
			iTradeCount = TradedResources(iResourceLoop);

			strText = strText .. iNumAvailable .. " ";
			strText = strText .. string.format("%s %s", pResource.IconString, Locale.ConvertTextKey(pResource.Description));
			-- Strategic Resources show X Total, Y Used even if Y is 0, but only if X > 0...

			if (iTradeCount > 0) then
				strExtraText = ": ";
				if (iNumAvailable ~= iTradeCount) then
					strExtraText = strExtraText .. Locale.ConvertTextKey("TXT_KEY_TP_LUXURY_RESOURCE_TOTAL", iNumAvailable);
					strExtraText = strExtraText .. ", ";
				end
				strExtraText = strExtraText .. Locale.ConvertTextKey("TXT_KEY_TP_LUXURY_RESOURCE_FROM_TRADE", iTradeCount);
			elseif (iTradeCount < 0) then
				strExtraText = ": ";
				strExtraText = strExtraText .. Locale.ConvertTextKey("TXT_KEY_TP_LUXURY_RESOURCE_TOTAL", iNumAvailable - iTradeCount);
				strExtraText = strExtraText .. ", ";
				strExtraText = strExtraText .. Locale.ConvertTextKey("TXT_KEY_TP_LUXURY_RESOURCE_TRADED", -1 * iTradeCount);
			end

			if (iNumAvailable > 1) then
				if (strExtraText == "") then strExtraText = ": " else strExtraText = strExtraText .. ", " end
				strExtraText = strExtraText .. Locale.ConvertTextKey("TXT_KEY_TP_LUXURY_RESOURCE_EXTRA", iNumAvailable - 1);
			end

			strText = strText .. strExtraText;

			table.insert(tKeys, Locale.ConvertTextKey(pResource.Description));
			tResources[Locale.ConvertTextKey(pResource.Description)] = strText;
		end
	end

	table.sort(tKeys);

	strText = "";
	for i,k in ipairs(tKeys) do
		if (i > 1) then
			strText = strText .. "[NEWLINE]";
		end
		strText = strText .. tResources[k];
	end

	-- print(strText);
	if(strText ~= "") then
		tipControlTable.TopPanelMouseover:SetHide(false);
		tipControlTable.TooltipLabel:SetText(strText);
	else
		tipControlTable.TopPanelMouseover:SetHide(true);
	end

    -- Autosize tooltip
    tipControlTable.TopPanelMouseover:DoAutoSize();

end

Events.SerialEventGameDataDirty.Add(UpdateLuxuryResourceDisplay);
Events.SerialEventTurnTimerDirty.Add(UpdateLuxuryResourceDisplay);
Events.SerialEventCityInfoDirty.Add(UpdateLuxuryResourceDisplay);

UpdateLuxuryResourceDisplay();
DoInitTooltipsLR();
